import moment from 'moment';
import { Metadata } from '@zeedhi/core';
import { OMInstance } from '@/services/OMInstance';
import { LoadingService, Table } from '@zeedhi/common';
import { TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { Highlight } from '@zeedhi/zd-highlight-common';
import { OmDialogService } from '@/services/OmDialogService';
import Prism from 'prismjs';
import 'prismjs/components/prism-markup-templating.min';
import 'prismjs/components/prism-php.min';

export class CollectionErrorsController {
    public filterDate: any;

    public isVisibleGrid = true;

    public isVisibleHighlights = false;

    public languageVar = 'php';

    public get addDatasourceScript() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'fifithRowHighlightScript');
    }

    public get organizationHeader() {
        const queries = Metadata.getInstances('ORGANIZATION_HEADER');
        return queries.find((query: any) => query.parent.name === 'ZdOrganizationHeader');
    }

    public get addDatasourceScriptCount() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'fifithRowScriptCount');
    }

    public get addDatasourceId() {
        const queries = Metadata.getInstances('ID');
        return queries.find((query: any) => query.parent.name === 'firstRowId');
    }

    public get addDatasourceOrganization() {
        const queries = Metadata.getInstances('ORGANIZATION');
        return queries.find((query: any) => query.parent.name === 'firstRowOrganization');
    }

    public get addDatasourceProduct() {
        const queries = Metadata.getInstances('PRODUCT');
        return queries.find((query: any) => query.parent.name === 'firstRowProduct');
    }

    public get addDatasourceVariableMeasure() {
        const queries = Metadata.getInstances('VARIABLE_MEASURE');
        return queries.find((query: any) => query.parent.name === 'FirstRowVariableMeasure');
    }

    public get addDatasourceService() {
        const queries = Metadata.getInstances('SERVICE');
        return queries.find((query: any) => query.parent.name === 'SecondRowService');
    }

    public get addDatasourceContractCode() {
        const queries = Metadata.getInstances('CONTRACT_CODE');
        return queries.find((query: any) => query.parent.name === 'SecondRowContractCode');
    }

    public get addDatasourceData() {
        const queries = Metadata.getInstances('DATA');
        return queries.find((query: any) => query.parent.name === 'SecondRowData');
    }

    public get addDatasourceLastSuccessfulCollect() {
        const queries = Metadata.getInstances('LAST_SUC_COLLECT');
        return queries.find((query: any) => query.parent.name === 'SecondRowDataLastSucCollect');
    }

    public get addDatasourceStringCon() {
        const queries = Metadata.getInstances('DSSTRINGCONEXAO');
        return queries.find((query: any) => query.parent.name === 'ThirdRowStringCon');
    }

    public get addDatasourceWallet() {
        const queries = Metadata.getInstances('IDUTWALLET');
        return queries.find((query: any) => query.parent.name === 'ThirdRowWallet');
    }

    public get addDatasourceCollector() {
        const queries = Metadata.getInstances('IDCOLETOR');
        return queries.find((query: any) => query.parent.name === 'ThirdRowCollector');
    }

    public get addDatasourceTpColeta() {
        const queries = Metadata.getInstances('IDTPCOLETA');
        return queries.find((query: any) => query.parent.name === 'ThirdRowTpColeta');
    }

    public get addDatasourceError() {
        const queries = Metadata.getInstances('ERROR');
        return queries.find((query: any) => query.parent.name === 'fourthRowError');
    }

    public get addDatasourceCollapseCardScriptCountPHP() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountPHP');
    }

    public get addDatasourceCollapseCardScriptPHP() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptPHP');
    }

    public get addDatasourceCollapseCardScriptCountSQL() {
        const queries = Metadata.getInstances('SCRIPT_COUNT');
        return queries.find((query: any) => query.parent.name === 'collapseCardScriptCountSQL');
    }

    public get addDatasourceCollapseCardScriptSQL() {
        const queries = Metadata.getInstances('SCRIPT');
        return queries.find((query: any) => query.parent.name === 'collapseCardHighlightScriptSQL');
    }

    public async organizationGridRowClick(params) {
        this.isVisibleGrid = false;
        const fifthTable = Metadata.getInstance<Table>('fifthRow');
        const thirdTable = Metadata.getInstance<Table>('thirdRow');
        const lastSucCollect = Metadata.getInstance<Table>('SecondRowDataLastSucCollect');
        const SQLCollapseCard = Metadata.getInstance<Table>('SQL_Sub_Scripts');
        const PHPCollapseCard = Metadata.getInstance<Table>('PHP_Sub_Scripts');
        try {
            LoadingService.show();
            this.addDatasourceId.text = params.row.ID;
            this.addDatasourceOrganization.text = params.row.NMORGANIZACAO;
            this.organizationHeader.text = params.row.NMORGANIZACAO;
            this.addDatasourceVariableMeasure.text = params.row.DSTPMEDIDASERV;
            this.addDatasourceProduct.text = params.row.PRODUCT_NAME;
            this.addDatasourceService.text = params.row.NMSERVORCA;
            this.addDatasourceContractCode.text = params.row.NRCONTRATOFCT;
            this.addDatasourceData.text = params.row.DTINCLUSAO;
            this.addDatasourceLastSuccessfulCollect.text = params.row.DTULTCOLETASUC;
            this.addDatasourceError.text = params.row.ERRO_COLETA;
            if (params.row.IDTPERRO === 'E' || params.row.IDTPERRO === 'A') {
                this.addDatasourceStringCon.text = params.row.DSSTRINGCONEXAO;
                this.addDatasourceCollector.text = params.row.IDCOLETOR;
                this.addDatasourceTpColeta.text = params.row.IDTPCOLETA;
                this.addDatasourceScriptCount.code = params.row.SCRIPT_COUNT;
                this.addDatasourceScript.code = params.row.SCRIPT_LOG;
                this.addDatasourceWallet.text = params.row.IDUTWALLET;
                if (params.row.IDUTWALLET === 'S') {
                    this.addDatasourceWallet.text = 'Sim';
                } else {
                    this.addDatasourceWallet.text = 'Não';
                }
                fifthTable.isVisible = true;
                thirdTable.isVisible = true;
                lastSucCollect.isVisible = true;
                PHPCollapseCard.isVisible = false;
                SQLCollapseCard.isVisible = false;
                this.isVisibleHighlights = false;
                if (params.row.IDTPSCRIPT === 'SQL') {
                    this.addDatasourceScript.language = 'sql';
                } else {
                    this.addDatasourceScript.language = 'php';
                }
            } else {
                this.addDatasourceCollapseCardScriptCountPHP.code = params.row.SCRIPT_COUNT_PHP;
                this.addDatasourceCollapseCardScriptPHP.code = params.row.SCRIPT_LOG_PHP;
                this.addDatasourceCollapseCardScriptCountSQL.code = params.row.SCRIPT_COUNT_SQL;
                this.addDatasourceCollapseCardScriptSQL.code = params.row.SCRIPT_LOG_SQL;
                fifthTable.isVisible = false;
                thirdTable.isVisible = false;
                lastSucCollect.isVisible = false;
                PHPCollapseCard.isVisible = true;
                SQLCollapseCard.isVisible = true;
                this.isVisibleHighlights = true;
            }
            LoadingService.hide();
        } catch (e) {
            LoadingService.hide();
            OmDialogService.showErrorBackend(e);
        }
    }

    get isVisibleHighLights(): boolean {
        return this.isVisibleHighlights;
    }

    get isVisibleDbVersionCard(): boolean {
        return !this.isVisibleGrid;
    }

    public closeButton() {
        this.isVisibleGrid = true;
    }

    public loadLanguage = this.loadLanguageAux.bind(this);

    public loadLanguageAux() {
        const query = this.addDatasourceScript;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptPHP = this.loadLanguageAuxCollapseCardScriptPHP.bind(this);

    public loadLanguageAuxCollapseCardScriptPHP() {
        const query = this.addDatasourceCollapseCardScriptPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountPHP = this.loadLanguageAuxCollapseCardCountPHP.bind(this);

    public loadLanguageAuxCollapseCardCountPHP() {
        const query = this.addDatasourceCollapseCardScriptCountPHP;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardScriptSQL = this.loadLanguageAuxCollapseCardScriptSQL.bind(this);

    public loadLanguageAuxCollapseCardScriptSQL() {
        const query = this.addDatasourceCollapseCardScriptSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCollapseCardCountSQL = this.loadLanguageAuxCollapseCardCountSQL.bind(this);

    public loadLanguageAuxCollapseCardCountSQL() {
        const query = this.addDatasourceCollapseCardScriptCountSQL;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageCount = this.loadLanguageAuxCount.bind(this);

    public loadLanguageAuxCount() {
        const query = this.addDatasourceScriptCount;
        if (query) {
            const { code, language } = query;
            return Prism.highlight(code, Prism.languages[language], language);
        }
        return '';
    }

    public loadLanguageExpression() {
        const { code, language } = Metadata.getInstance<Highlight>('expression');
        return Prism.highlight(code, Prism.languages[language], language);
    }

    public onCreatedCollectionErrorsGrid() {
        const today = moment().format('DD/MM/YYYY');
        this.filterDate = [today, today];
        const grid = OMInstance.getInstanceTekGrid('collectionErrors-grid');
        (grid.datasource as TekRestDatasource).dynamicFilter = {
            DTINCLUSAO: [{
                relation: 'AND',
                operation: 'BETWEEN',
                value: [today, today],
            }],
            IDTPERRO: [{
                relation: 'AND',
                operation: 'IN',
                value: ['E'],
            }],
        };
        grid.datasource.get();
    }

    public getConditionalCssClass(param: any) {
        if (param.row.IDTPERRO === 'E') return 'idtperro';
        if (param.row.IDTPERRO === 'A') return 'idtperroWarningCollect';
        return 'idtperroWarning';
      }
}
